@@ -4,6 +4,7 @@ Routes and sub-resources for the /emulations resource
 import base64
 import requests
 import json
+from requests import get
 from flask import Blueprint, jsonify, request
 import csle_common.constants.constants as constants
 from csle_common.metastore.metastore_facade import MetastoreFacade
@@ -252,112 +253,145 @@ def get_sdn_switches_of_execution(emulation_id: int, exec_id: int):
         return authorized
 
     em = MetastoreFacade.get_emulation(id=emulation_id)
+
     response_data = {}
     if em is not None and em.sdn_controller_config is not None:
-        executions = MetastoreFacade.list_emulation_executions_for_a_given_emulation(emulation_name=em.name)
-        for exec in executions:
-            if int(exec.ip_first_octet) == int(exec_id):
+        exec = MetastoreFacade.get_emulation_execution(ip_first_octet=exec_id, emulation_name=em.name)
+        local_port = api_constants.MGMT_WEBAPP.RYU_TUNNEL_BASE_PORT + exec.ip_first_octet
+        if exec.emulation_env_config.sdn_controller_config.container.get_ips()[0] \
+                not in api_constants.MGMT_WEBAPP.RYU_TUNNELS_DICT:
+            try:
+                EmulationEnvController.create_ssh_tunnel(
+                    tunnels_dict=api_constants.MGMT_WEBAPP.RYU_TUNNELS_DICT,
+                    local_port=local_port,
+                    remote_port=exec.emulation_env_config.sdn_controller_config.controller_web_api_port,
+                    remote_ip=exec.emulation_env_config.sdn_controller_config.container.get_ips()[0])
+            except Exception:
+                local_port = local_port + 100
+                EmulationEnvController.create_ssh_tunnel(
+                    tunnels_dict=api_constants.MGMT_WEBAPP.RYU_TUNNELS_DICT,
+                    local_port=local_port,
+                    remote_port=exec.emulation_env_config.sdn_controller_config.controller_web_api_port,
+                    remote_ip=exec.emulation_env_config.sdn_controller_config.container.get_ips()[0])
+        else:
+            tunnel_thread_dict = api_constants.MGMT_WEBAPP.RYU_TUNNELS_DICT[
+                exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]]
+            response = get(f'{constants.HTTP.HTTP_PROTOCOL_PREFIX}{constants.COMMON.LOCALHOST}:'
+                           f'{local_port}')
+            if response.status_code != constants.HTTPS.OK_STATUS_CODE:
+                tunnel_thread_dict[api_constants.MGMT_WEBAPP.THREAD_PROPERTY].shutdown()
+                del api_constants.MGMT_WEBAPP.RYU_TUNNELS_DICT[
+                    exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]]
+                EmulationEnvController.create_ssh_tunnel(
+                    tunnels_dict=api_constants.MGMT_WEBAPP.RYU_TUNNELS_DICT,
+                    local_port=local_port,
+                    remote_port=exec.emulation_env_config.sdn_controller_config.controller_web_api_port,
+                    remote_ip=exec.emulation_env_config.sdn_controller_config.container.get_ips()[0])
+
+        if int(exec.ip_first_octet) == int(exec_id):
+            response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                    f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                    f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                    f"{ryu_constants.RYU.STATS_SWITCHES_RESOURCE}")
+            switches = json.loads(response.content)
+            switches_dicts = []
+            for dpid in switches:
                 response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
                                         f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
                                         f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                        f"{ryu_constants.RYU.STATS_SWITCHES_RESOURCE}")
-                switches = json.loads(response.content)
-                switches_dicts = []
-                for dpid in switches:
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_DESC_RESOURCE}/{dpid}")
-                    sw_dict = {}
-                    sw_dict[api_constants.MGMT_WEBAPP.DPID_PROPERTY] = dpid
-                    sw_dict[api_constants.MGMT_WEBAPP.DESC_PROPERTY] = json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_FLOW_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.FLOWS_PROPERTY] = json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_AGGREGATE_FLOW_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.AGG_FLOWS_PROPERTY] = json.loads(response.content)[str(dpid)][0]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_TABLE_RESOURCE}/{dpid}")
-                    tables = json.loads(response.content)[str(dpid)]
-                    tables = list(filter(lambda x: x[api_constants.MGMT_WEBAPP.ACTIVE_COUNT_PROPERTY] > 0, tables))
-                    filtered_table_ids = list(map(lambda x: x[api_constants.MGMT_WEBAPP.TABLE_ID_PROPERTY], tables))
-                    sw_dict[api_constants.MGMT_WEBAPP.TABLES_PROPERTY] = tables
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_TABLE_FEATURES_RESOURCE}/{dpid}")
-                    tablefeatures = json.loads(response.content)[str(dpid)]
-                    tablefeatures = list(filter(
-                        lambda x: x[api_constants.MGMT_WEBAPP.TABLE_ID_PROPERTY] in filtered_table_ids, tablefeatures))
-                    sw_dict[api_constants.MGMT_WEBAPP.TABLE_FEATURES_PROPERTY] = tablefeatures
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_PORT_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.PORT_STATS_PROPERTY] = json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_PORT_DESC_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.PORT_DESCS_PROPERTY] = json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_QUEUE_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.QUEUES_PROPERTY] = json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_QUEUE_CONFIG_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.QUEUE_CONFIGS_PROPERTY] = \
-                        json.loads(response.content)[str(dpid)][0]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_GROUP_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.GROUPS_PROPERTY] = json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_GROUP_DESC_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.GROUP_DESCS_PROPERTY] = json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_GROUP_FEATURES_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.GROUP_FEATURES_PROPERTY] = \
-                        json.loads(response.content)[str(dpid)][0]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_METER_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.METERS_PROPERTY] = json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_METER_CONFIG_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.METER_CONFIGS_PROPERTY] = \
-                        json.loads(response.content)[str(dpid)]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_METER_FEATURES_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.METER_FEATURES_PROPERTY] = \
-                        json.loads(response.content)[str(dpid)][0]
-                    response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
-                                            f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
-                                            f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
-                                            f"{ryu_constants.RYU.STATS_ROLE_RESOURCE}/{dpid}")
-                    sw_dict[api_constants.MGMT_WEBAPP.ROLES_PROPERTY] = json.loads(response.content)[str(dpid)][0]
-                    switches_dicts.append(sw_dict)
-                response_data = switches_dicts
+                                        f"{ryu_constants.RYU.STATS_DESC_RESOURCE}/{dpid}")
+                sw_dict = {}
+                sw_dict[api_constants.MGMT_WEBAPP.DPID_PROPERTY] = dpid
+                sw_dict[api_constants.MGMT_WEBAPP.DESC_PROPERTY] = json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_FLOW_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.FLOWS_PROPERTY] = json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_AGGREGATE_FLOW_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.AGG_FLOWS_PROPERTY] = json.loads(response.content)[str(dpid)][0]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_TABLE_RESOURCE}/{dpid}")
+                tables = json.loads(response.content)[str(dpid)]
+                tables = list(filter(lambda x: x[api_constants.MGMT_WEBAPP.ACTIVE_COUNT_PROPERTY] > 0, tables))
+                filtered_table_ids = list(map(lambda x: x[api_constants.MGMT_WEBAPP.TABLE_ID_PROPERTY], tables))
+                sw_dict[api_constants.MGMT_WEBAPP.TABLES_PROPERTY] = tables
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_TABLE_FEATURES_RESOURCE}/{dpid}")
+                tablefeatures = json.loads(response.content)[str(dpid)]
+                tablefeatures = list(filter(
+                    lambda x: x[api_constants.MGMT_WEBAPP.TABLE_ID_PROPERTY] in filtered_table_ids, tablefeatures))
+                sw_dict[api_constants.MGMT_WEBAPP.TABLE_FEATURES_PROPERTY] = tablefeatures
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_PORT_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.PORT_STATS_PROPERTY] = json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_PORT_DESC_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.PORT_DESCS_PROPERTY] = json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_QUEUE_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.QUEUES_PROPERTY] = json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_QUEUE_CONFIG_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.QUEUE_CONFIGS_PROPERTY] = \
+                    json.loads(response.content)[str(dpid)][0]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_GROUP_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.GROUPS_PROPERTY] = json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_GROUP_DESC_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.GROUP_DESCS_PROPERTY] = json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_GROUP_FEATURES_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.GROUP_FEATURES_PROPERTY] = \
+                    json.loads(response.content)[str(dpid)][0]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_METER_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.METERS_PROPERTY] = json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_METER_CONFIG_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.METER_CONFIGS_PROPERTY] = \
+                    json.loads(response.content)[str(dpid)]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_METER_FEATURES_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.METER_FEATURES_PROPERTY] = \
+                    json.loads(response.content)[str(dpid)][0]
+                response = requests.get(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}"
+                                        f"{exec.emulation_env_config.sdn_controller_config.container.get_ips()[0]}:"
+                                        f"{exec.emulation_env_config.sdn_controller_config.controller_web_api_port}"
+                                        f"{ryu_constants.RYU.STATS_ROLE_RESOURCE}/{dpid}")
+                sw_dict[api_constants.MGMT_WEBAPP.ROLES_PROPERTY] = json.loads(response.content)[str(dpid)][0]
+                switches_dicts.append(sw_dict)
+            response_data = {}
+            response_data[api_constants.MGMT_WEBAPP.SWITCHES_SUBRESOURCE] = switches_dicts
+            response_data[api_constants.MGMT_WEBAPP.SDN_CONTROLLER_LOCAL_PORT] = local_port
     response = jsonify(response_data)
     response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
     return response, constants.HTTPS.OK_STATUS_CODE
